/* -*- mode:prolog; -*-
 ******************************************************************
 * $Id$
 * Copyright (C) 2008 by INRIA 
 * Author: Eric de la Clergerie <Eric.De_La_Clergerie@inria.fr>
 * ----------------------------------------------------------------
 *
 *  regexp2fsa.pl -- Compilateur Regexp vers FSA
 *
 * ----------------------------------------------------------------
 * Description
 *
 * Petit exemple de compilateur d'expressions rgulire en automates 
 * tats finis
 *
 * ----------------------------------------------------------------
 */

%% Import de la bibliothque DyALog pour l'impression
:-require 'format.pl'.

%% Dclaration des oprateurs pour construire les regexp
:-op(yf,['@?'],500).
:-op(yf,['@+'],500).
:-op(yf,['@*'],500).

%% ** Spcifique DyALog **
%% Pour ne pas tabuler le prdicat regexp2fsa
%% et avoir un comportement standard a la Prolog
:-rec_prolog regexp2fsa/4.

%% Dconstruire les constructeurs de regexp

%% Concatnation
regexp2fsa(Name,(E1,E2),S1,S2) :-
	regexp2fsa(Name,E1,S1,S3),
	regexp2fsa(Name,E2,S3,S2)
	.

%% Alternative
regexp2fsa(Name,(E1;E2),S1,S2) :-
	regexp2fsa(Name,E1,S1,S2),
	regexp2fsa(Name,E2,S1,S2)
	.

%% Optionnel
regexp2fsa(Name,E @?,S1,S2) :-
	regexp2fsa(Name,E,S1,S2),
	emit_trans(Name,S1,S2,true)
	.

%% toile de Kleene
regexp2fsa(Name,E @*,S1,S2) :-
	regexp2fsa(Name,E,S1,S1),
	regexp2fsa(Name,true,S1,S2)
	.

%% toile de Kleene stricte
regexp2fsa(Name,E @+,S1,S2) :-
	regexp2fsa(Name,E,S1,S1),
	regexp2fsa(Name,true,S1,S2)
	.

%% Expressions de base
regexp2fsa(Name,A,S1,S2) :-
	domain(A,[c(X),range(D),true]),
	(S2 is S1+1 xor true),
	emit_trans(Name,S1,S2,A).

%% Prdicat prolog-like pour l'mission des transitions
:-rec_prolog emit_trans/4.

emit_trans(Name,S1,S2,A) :-
	format('trans(~w,~w,~w,~q).\n',[Name,S1,S2,A])
	.

%% Le prdicat base de donnes pour accder aux regexp
:-extensional rx/2.

%% La requte chapeau pour la compilation
?-rx(Name,RX),
  %% mission d'un petit header
  format('%% Compilation de la regexp ~w\n\n',[Name]),
  %% traitement de l'expression rgulire
  regexp2fsa(Name,RX,0,Final),
  %% mission des tats initiaux et finaux
  format('init_state(~w,~w).\n',[Name,0]),
  format('final_state(~w,~w).\n',[Name,Final]),
  fail.
